@@ -23,17 +23,14 @@
 # python 3 headers, required if submitting to Ansible
 
 import syspassclient
+
 from colorama import init, Fore, Style
-
 init(autoreset=True)
-import warnings
-from urllib3.exceptions import HTTPWarning, InsecureRequestWarning
 
-warnings.simplefilter("ignore", HTTPWarning)
-warnings.simplefilter("ignore", InsecureRequestWarning)
 import requests
+requests.urllib3.disable_warnings()
 
-requests.urllib3.disable_warnings(category=InsecureRequestWarning)
+from json.decoder import JSONDecodeError
 
 from yaml import dump
 
@@ -42,9 +39,7 @@ try:
 except ImportError:
     from yaml import Loader, Dumper
 
-
 import threading
-
 lock = threading.Lock()
 
 
@@ -65,17 +60,12 @@ class SyspassClient(syspassclient.Object, syspassclient.Utils, metaclass=Singlet
         syspassclient.Utils.__init__(self)
 
         # Private property's
-        self.__authToken = None
-        self.__api_url = None
-        self.__tokenPass = None
         self.__r_id = None
 
         # Initialize
         self.__config = None
         self.config = syspassclient.config
-        self.api_url = self.config.api_url
-        self.authToken = self.config.authToken
-        self.tokenPass = self.config.tokenPass
+
         self.debug = self.config.debug
         self.debug_level = self.config.debug_level
         self.verbose = self.config.verbose
@@ -85,135 +75,42 @@ class SyspassClient(syspassclient.Object, syspassclient.Utils, metaclass=Singlet
 
     @property
     def config(self):
+        """
+        Store config object, as a property
+
+        :return: The configuration object
+        :rtype: Config or None
+        """
         return self.__config
 
     @config.setter
     def config(self, value=None):
-        if value is None and self.config is None:
-            return
+        """
+        Set the ``config`` property value
+
+        :param value: a config instance or None
+        :type value: Config or None
+        :raise TypeError: When ``config`` value is not a Config instance or None
+        """
         if value is not None and not isinstance(value, syspassclient.Config):
-            raise TypeError('"config" value must be a a syspassclient.Config instance')
+            raise TypeError('"config" value must be a syspassclient.Config instance')
         if self.config != value:
             self.__config = value
 
-    @property
-    def api_url(self):
+    def make_post_request(self, data=None, verify=None):
         """
-        The url of the SysPass API
-
-        :return: the url of the SysPass API or None if not set
-        :rtype: str or None
-        """
-        return self.__api_url
-
-    @api_url.setter
-    def api_url(self, api_url=None):
-        """
-        Set ``api_url`` property
-
-        :param api_url: A SysPass API Key
-        :type api_url: str or None
-        :raise TypeError: if ``api_url`` is not a str of None
-        """
-
-        if api_url is None:
-            if self.__api_url is not None:
-                self.__api_url = None
-            return
-
-        self.is_str_or_raise(api_url)
-        self.is_url_or_raise(api_url)
-
-        if self.__api_url != api_url:
-            self.__api_url = api_url
-
-    # @property
-    # def authToken(self):
-    #     """
-    #     Whether the application will paint directly on the widget.
-    #
-    #     :return: True or False
-    #     :rtype: bool
-    #     """
-    #     return self.config.authToken
-
-    # @authToken.setter
-    # def authToken(self, authToken=None):
-    #     """
-    #     Set ``api_key`` property
-    #
-    #     :param authToken: A SysPass API Key
-    #     :type authToken: str or None
-    #     :raise TypeError: if ``api_key`` is not a str of None
-    #     """
-    #     if authToken is None or authToken == "":
-    #         if self.__authToken != authToken:
-    #             self.__authToken = None
-    #         return
-    #
-    #     self.is_str_or_raise(authToken)
-    #     self.is_ascii_or_raise(authToken)
-    #
-    #     if self.__authToken != authToken:
-    #         self.__authToken = authToken
-
-    # @property
-    # def tokenPass(self):
-    #     """
-    #     Whether the application will paint directly on the widget.
-    #
-    #     :return: True or False
-    #     :rtype: bool
-    #     """
-    #     # if self.config.rw:
-    #     return self.config.tokenPass
-    #
-    # @tokenPass.setter
-    # def tokenPass(self, tokenPass=None):
-    #     """
-    #     Set ``api_acc_tokpwd`` property
-    #
-    #     :param tokenPass: A SysPass API Key
-    #     :type tokenPass: str or None
-    #     :raise TypeError: if ``api_acc_tokpwd`` is not a str of None
-    #     """
-    #     if tokenPass is None or tokenPass == "":
-    #         if self.__tokenPass != tokenPass:
-    #             self.__tokenPass = None
-    #         return
-    #
-    #     if tokenPass is not None:
-    #         self.is_str_or_raise(tokenPass)
-    #         self.is_ascii_or_raise(tokenPass)
-    #
-    #     if self.__tokenPass != tokenPass:
-    #         self.__tokenPass = tokenPass
-
-    def make_post_request(self, data=None, verify=False):
-        """
-        make the request by 'request' function.
+        Make the request ith the 'requests' python module.
 
         :param data: the data it will be convert on json
         :type data: dict
-        :param verify: ssl certificate verification
+        :param verify: SSL certificate verification
         :type verify: bool
         :return: the request.post return
         :rtype: dict
         """
-        # if self.debug and self.debug_level > 0:
-        #     class_name = "{0} ".format(self.__class__.__name__.upper())
-        #     action_name = "| {1} | {0} ".format("request >".lower(), data['method'])
-        #
-        #     sys.stdout.write("\u001b[33m")
-        #     sys.stdout.write("{0}".format(class_name))
-        #     sys.stdout.write("\u001b[0m")
-        #     sys.stdout.write("{0}".format(action_name))
-        #     sys.stdout.write("\n")
-        #     sys.stdout.write("{0}".format(dump(data)))
-        #     sys.stdout.write("\u001b[0m")
-        #     sys.stdout.write("\n")
-        #
-        #     sys.stdout.flush()
+        if verify is None:
+            verify = self.config.verify_ssl
+
         if self.verbose:
             print(Fore.YELLOW + Style.BRIGHT + "{0}: ".format(self.__class__.__name__.upper()), end='')
             print(Fore.WHITE + Style.BRIGHT + "{0} ".format(data['method']))
@@ -222,16 +119,14 @@ class SyspassClient(syspassclient.Object, syspassclient.Utils, metaclass=Singlet
                     print(Fore.GREEN + Style.BRIGHT + "> " + Fore.RESET + Style.RESET_ALL + str(line))
             print(Fore.RESET + Style.RESET_ALL, end='')
 
-        request = requests.post(url=self.api_url, json=data, verify=verify)
-        # print(request.text)
-        from json.decoder import JSONDecodeError
+        request = requests.post(url=self.config.api_url, json=data, verify=verify)
+
         try:
             return request.json()
-        except JSONDecodeError:
+        except JSONDecodeError:  # pragma: no cover
+            # In case we don't receive json thing
             return request.text
 
-        # return requests.post(url=self.api_url, json=data, verify=verify).json()
-
     def increase_request_id(self, increment=1):
         """
         In case of multi request, request ID must be increase for permit to the API work with.
@@ -249,6 +144,21 @@ class SyspassClient(syspassclient.Object, syspassclient.Utils, metaclass=Singlet
         self.r_id += increment
 
     def generate_json(self, **parameters):
+        """
+        Internal function it generate a valid request in JSON format
+
+        That function clean the request and mitigate python reserved name, like **Global**, **Id**, **Pass**
+
+        For paramaters list take a look on SysPass API:
+        * https://syspass-doc.readthedocs.io/en/3.0/application/api.html
+        * https://syspass-doc.readthedocs.io/en/3.1/application/api.html
+
+        :param parameters: The parameters pass to syspass API
+        :type parameters: keys
+        :return: The parameters format in valid JSON 2.0
+        :rtype: str
+        :raise KeyError: When parameters dont contain a key name "method"
+        """
 
         if "method" not in parameters:
             raise (KeyError('"parameters" must have a key name "method"'))
@@ -272,29 +182,44 @@ class SyspassClient(syspassclient.Object, syspassclient.Utils, metaclass=Singlet
             ugid = parameters["ugid"]
             del parameters["ugid"]
             parameters["id"] = ugid
+
         if "cid" in parameters:
             cid = parameters["cid"]
             del parameters["cid"]
             parameters["id"] = cid
+
         if "tagid" in parameters:
             tagid = parameters["tagid"]
             del parameters["tagid"]
             parameters["id"] = tagid
+
         if "account_id" in parameters:
             account_id = parameters["account_id"]
             del parameters["account_id"]
             parameters["id"] = account_id
 
+        # Small clean up
         for key, value in dict(parameters).items():
-            if value is None:
+            if value is None:  # pragma: no cover
                 del parameters[key]
-            if value is []:
+            if value is []:  # pragma: no cover
                 del parameters[key]
 
-        data = {"jsonrpc": "2.0", "method": method, "params": parameters, "id": self.r_id}
-        return data
+        return {"jsonrpc": "2.0",
+                "method": method,
+                "params": parameters,
+                "id": self.r_id
+                }
 
     def print_returned_value(self, req):
+        """
+        Internal function to print returned value.
+
+        Printed values depends of ``verbose`` and ``verbose_level`` properties
+
+        :param req: The return of requests module as a dict
+        :type req: dict
+        """
         if self.verbose:
             if self.verbose_level > 1:
                 print(Fore.YELLOW + Style.BRIGHT + "{0}: ".format(self.__class__.__name__.upper()), end='')
@@ -392,16 +317,36 @@ class SyspassClient(syspassclient.Object, syspassclient.Utils, metaclass=Singlet
         self.print_returned_value(req)
 
         # look_for_error(data, req)
+        return self.controlled_match_all(req=req, matchall=matchall, text=text)
 
-        if 'result' in req and 'count' in req['result'] and req['result']['count'] > 0:
-            if matchall is None:
+    @staticmethod
+    def controlled_match_all(req=None, matchall=None, text=None):
+        """
+        Internal function it control match_all mechanic
+
+        :param req: The requests python module return
+        :type req: dict
+        :param matchall: If True the returned value contain all it match with the request
+        :param text:
+        :return:
+        """
+        if matchall is None:
+            matchall = False
+
+        if type(matchall) != bool:
+            raise TypeError("'matchall' must be a bool type or None")
+
+        if type(req) != dict:
+            raise TypeError("'req' must be a dict type")
+
+        if 'result' in req and 'count' in req['result'] and req['result']['count'] > 0 and 'result' in req['result']:
+            if matchall is False:
                 for res in req['result']['result']:
-                    if res['name'] == text:
-                        return req['result']['result'][0]['id']
+                    if res['name'].upper() == text.upper():
+                        return res['id']
             else:
-                return req['result']['result']
-        # elif 'error' in req:
-        #     raise AnsibleError('AccountSearch Error : %s' % req)
+                if 'id' in req['result']['result'][0]:
+                    return req['result']['result'][0]['id']
         else:
             return None
 
@@ -483,7 +428,7 @@ class SyspassClient(syspassclient.Object, syspassclient.Utils, metaclass=Singlet
 
         if 'result' in req and 'result' in req['result'] and len(req['result']['result']) > 0:
             return req["result"]["result"]
-        else:
+        else:  # pragma: no cover
             return None
 
     def account_viewpass(self, method="account/viewPass", authToken=None, tokenPass=None, account_id=None,
@@ -625,15 +570,11 @@ class SyspassClient(syspassclient.Object, syspassclient.Utils, metaclass=Singlet
         """
         if authToken is None:
             authToken = self.config.authToken
-        if authToken is None:
-            raise TypeError("'authToken' must be set")
 
         if tokenPass is None:
             tokenPass = self.config.tokenPass
 
-        if tokenPass is None:
-            raise TypeError("'tokenPass' must be set")
-        if expireDate == "":
+        if expireDate == "":  # pragma: no cover
             expireDate = None
 
         self.look_for_args(
@@ -683,10 +624,10 @@ class SyspassClient(syspassclient.Object, syspassclient.Utils, metaclass=Singlet
 
         if req["result"]["itemId"] > 0:
             return req["result"]["itemId"]
-        else:
+        else:  # pragma: no cover
             return None
 
-    def category_search(self, method="category/search", authToken=None, text=None, count=None):
+    def category_search(self, method="category/search", authToken=None, text=None, count=1000):
         """
         Searches syspass category.
         text is the keyword.
@@ -709,13 +650,16 @@ class SyspassClient(syspassclient.Object, syspassclient.Utils, metaclass=Singlet
 
         # We got a response
 
-        if 'result' in req and 'count' in req['result'] and req['result']['count'] > 0:
+        # print('################################################')
+        # for res in req['result']['result']:
+        #     print(dump(res))
+        # print('################################################')
+
+        if 'result' in req and 'count' in req['result'] and req['result']['count'] > 0 and 'result' in req['result']:
             for res in req['result']['result']:
-                if res['name'] == text:
-                    return req['result']['result'][0]['id']
-        # elif 'error' in req:
-        #     raise AnsibleError('CategorySearch Error : %s' % (req['error']))
-        else:
+                if res['name'].upper() == text.upper():
+                    return res['id']
+        else:  # pragma: no cover
             return None
 
     def category_create(self, method="category/create", authToken=None, name=None, description=None):
@@ -751,12 +695,10 @@ class SyspassClient(syspassclient.Object, syspassclient.Utils, metaclass=Singlet
         self.print_returned_value(req)
 
         # look_for_error(data, req)
-        if 'result' in req and 'itemId' in req['result'] and req['result']['itemId'] > 0:
-            if req["result"]['itemId'] is None:
-                return self.category_search(text=name)
-            else:
-                return req["result"]['itemId']
-        else:
+        if 'result' in req and 'itemId' in req['result'] and \
+                req['result']['itemId'] > 0 and req["result"]['itemId'] is not None:
+            return req["result"]['itemId']
+        else:  # pragma: no cover
             return self.category_search(text=name)
 
     def category_delete(self, method="category/delete", authToken=None, cid=None):
@@ -784,7 +726,6 @@ class SyspassClient(syspassclient.Object, syspassclient.Utils, metaclass=Singlet
 
         self.look_for_error(data, req)
 
-        # print(req)
         if req["result"]["resultCode"] == 0:
             return req["result"]
         # else:
@@ -820,9 +761,8 @@ class SyspassClient(syspassclient.Object, syspassclient.Utils, metaclass=Singlet
             for res in req['result']['result']:
                 if res['name'].upper() == text.upper():
                     return req['result']['result'][0]['id']
-        # elif 'error' in req:
-        #     raise AnsibleError('ClientSearch Error : %s' % (req['error']))
-        else:
+
+        else:  # pragma: no cover
             return None
 
     def client_create(self, method="client/create", authToken=None, name=None, description=None, Global=False):
@@ -979,9 +919,10 @@ class SyspassClient(syspassclient.Object, syspassclient.Utils, metaclass=Singlet
 
         self.look_for_error(data, req)
 
-        if 'result' in req and 'result' in req['result'] and len(req['result']['result']) > 0:
-            return req["result"]["result"]
-        else:
+        if 'result' in req and 'result' in req['result'] and len(req['result']['result']) > 0 and \
+                'name' in req['result']['result']:
+            return req["result"]["result"]['name']
+        else:  # pragma: no cover
             return None
 
     def tag_create(self, method='tag/create', authToken=None, name=None):
@@ -1021,7 +962,6 @@ class SyspassClient(syspassclient.Object, syspassclient.Utils, metaclass=Singlet
 
         return self.tag_search(text=name)
 
-
         # if self.config.rw:
         #     return self.tag_search(text=name)
         # else:
@@ -1030,7 +970,6 @@ class SyspassClient(syspassclient.Object, syspassclient.Utils, metaclass=Singlet
         #         raise ValueError('impossible to creat the tag')
         #     return to_return
 
-
     def tag_delete(self, method='tag/delete', authToken=None, tagid=None):
         """
         tag/delete
@@ -1065,7 +1004,7 @@ class SyspassClient(syspassclient.Object, syspassclient.Utils, metaclass=Singlet
 
         if 'result' in req and 'resultCode' in req['result']:
             return req['result']['resultCode']
-        else:
+        else:  # pragma: no cover
             return None
         # if req['result']['resultCode'] == 0:
         #     return req['result']
@@ -1133,15 +1072,36 @@ class SyspassClient(syspassclient.Object, syspassclient.Utils, metaclass=Singlet
         req = self.make_post_request(data)
         self.print_returned_value(req)
 
-        if 'result' in req and 'count' in req['result'] and req['result']['count'] > 0:
+        if 'result' in req and 'count' in req['result'] and req['result']['count'] > 0 and 'result' in req['result']:
             for res in req['result']['result']:
                 if res['name'].upper() == text.upper():
-                    return req['result']['result'][0]['id']
+                    # return req['result']['result'][0]['id']
+                    return res['id']
 
-        else:
+        else:  # pragma: no cover
             self.look_for_error(data, req)
             return None
 
+    # SYSPASSCLIENT: userGroup/search
+    # > id: 27
+    # > jsonrpc: '2.0'
+    # > method: userGroup/search
+    # > params:
+    # >   authToken: b865b432f553960c1b1a8742a21f54885401b9abb37649d81d0d065f37183eb4
+    # >   text: FOR_TEST
+    # DD_SYSPASS_CLIENT_API_URL: https://prepsyspass.ddprep.digdeo.net/api.php
+    # < id: 27
+    # < jsonrpc: '2.0'
+    # < result:
+    # <   count: 1
+    # <   itemId: 0
+    # <   result:
+    # <   - description: a UserGroup for test
+    # <     id: 1412
+    # <     name: FOR_TEST
+    # <     users: null
+    # <   resultCode: 0
+    # <   resultMessage: null
     def user_group_delete(self,
                           method='userGroup/delete', authToken=None, ugid=None):
         """
@@ -1171,7 +1131,7 @@ class SyspassClient(syspassclient.Object, syspassclient.Utils, metaclass=Singlet
 
         if 'result' in req and 'resultCode' in req['result'] and req['result']['resultCode'] == 0:
             return req['result']
-        else:
+        else:  # pragma: no cover
             return None
             # look_for_error(data, req)
 
